/*****************************************************************************
*
*   Analog Devices, Inc.
*   ADSP-BF537 EZ-Kit Lite
*   Networked Audio Server
*
******************************************************************************
*
*   File        :   audio.c
*
*   Description :   This file contains routines for the main audio
*                   thread.
*
******************************************************************************
*   Revision History
******************************************************************************
*   date        rev     by      description
*-----------------------------------------------------------------------------
*   02.27.07    01.02h  KCK     Original release.
*****************************************************************************/

#include <cdefBF537.h>
#include <lwip/sockets.h>
#include <lwip/inet.h>
#include <lwip/cglobals.h>
#include <lwip/kernel_abs.h>
#include <lwip/ADI_TOOLS_IOEThreadType.h>
#include <lwip/ADI_ETHER_BF537.h>

#include <services/services.h>      // system service includes
#include <drivers/adi_dev.h>        // device manager includes
#include <drivers/sport/adi_sport.h>// sport driver includes

#include "system.h"
#include "audio.h"
#include "boot.h"
#include "queue.h"


#undef  USE_DEFERRED_CALLBACKS  // enables deferred callbacks

#pragma align 4
section ("L1_data_b") ADI_DEV_1D_BUFFER   audioBuffer[AUDIO_BUFFER_COUNT];                // the buffer descriptors
#pragma align 4
section ("L1_data_b") u8 audioData[AUDIO_BUFFER_COUNT][AUDIO_BUFFER_LEN];// the actual data buffers

u32                 audioPacketCount = 0;

section ("L1_data") ADI_DEV_DEVICE_HANDLE   audioInHandle;   // handle to the audio input driver


int     audioSocketID;
struct  sockaddr_in clientAudioSocketAddress;
struct  sockaddr_in serverAudioSocketAddress;


volatile u32 control_thread = 0;
queue_t audioQueue;
queueEntry_t audioQueueEntry[AUDIO_BUFFER_COUNT];





int
AudioNetworkConnect(void)
    {

    // create an Audio UPD socket (SOCK_DGRAM) with Internet Protocol Family (PF_INET)
    audioSocketID = socket(PF_INET, SOCK_DGRAM, 0);
    
    if ( audioSocketID < 0 )
    {
        #ifdef DEBUG_PRINT_ENABLE
        printf("socket call failed\n");
        #endif
        return(-1);
    }

    memset(&serverAudioSocketAddress, 0, sizeof(struct sockaddr_in));
    serverAudioSocketAddress.sin_family = AF_INET;
    serverAudioSocketAddress.sin_addr.s_addr = inet_addr(SERVER_IP_ADDR);
    serverAudioSocketAddress.sin_port = htons(SERVER_AUDIO_PORT);


    // Bind the UDP socket to the port SERVER_AUDIO_PORT and to the current
    // machines IP address (Its defined by SERVER_AUDIO_PORT).
    // Once bind is successful for UDP sockets application can operate
    // on the socket descriptor for sending or receiving data.


    if (bind(audioSocketID, (struct sockaddr *)&serverAudioSocketAddress, sizeof(struct sockaddr_in)) == -1)
        {
        #ifdef DEBUG_PRINT_ENABLE
        printf("Bind to Port Number %d ,IP address %s failed\n",SERVER_AUDIO_PORT, SERVER_IP_ADDR);
        #endif
        close(audioSocketID);
        return(-1);
        }

    
    memset(&clientAudioSocketAddress, 0, sizeof(struct sockaddr_in));
    clientAudioSocketAddress.sin_family = AF_INET;
    clientAudioSocketAddress.sin_addr.s_addr = inet_addr(CLIENT_IP_ADDR);
    clientAudioSocketAddress.sin_port = htons(CLIENT_AUDIO_PORT);

    if (connect(audioSocketID, (struct sockaddr *)&clientAudioSocketAddress, sizeof(struct sockaddr)) < 0)
        {
        #ifdef DEBUG_PRINT_ENABLE
        printf("Connect to client failed\n");
        #endif
        close(audioSocketID);
        return(-1);
        }
    return(0);
    }


// thread creation block to send audio over the network
static VDK_ThreadCreationBlock audioSendTCB = 
    {
    kaudiosend,         // template ID
    (VDK_ThreadID)0,    // thread ID
    0,                  // thread stack size
    (VDK_Priority)0,    // thread priority
    0,                  // user data pointer
    0                   // thread template
    };



static void AudioInCallback(
    void *AppHandle,
    u32  Event,
    void *pArg)
    {
    int result;
    ADI_DEV_1D_BUFFER *pBuffer = (ADI_DEV_1D_BUFFER *) pArg;
        
    switch (Event)
        {

        // CASE (DMA buffer processed)
        case ADI_DEV_EVENT_BUFFER_PROCESSED:

            // put arriving packet on send queue
            QueuePush(&audioQueue, pArg);
            VDK_C_ISR_PostSemaphore(kaudioPacketAvailable);

            *pPORTFIO_TOGGLE = 0x0400;  // toggle LED 5
        break;

        }
    }


char keepAliveMessage;

void
audio_RunFunction(void **inPtr)
    {
    int     result;
    int     i;                  // counter

    *pPORTFIO_SET   = 0x0040; // set LED 1


    if(AudioNetworkConnect() > -1)
        {

        // client is connected, start audio transfer
        #ifdef DEBUG_PRINT_ENABLE
        printf("Audio connection established\n");
        #endif


        // prepare buffers for audio input driver to store the data
        for (i = 0; i < AUDIO_BUFFER_COUNT; i++) 
            {
            audioBuffer[i].Data                 = &audioData[i][0];
            audioBuffer[i].ElementCount         = AUDIO_ELEMENT_COUNT;
            audioBuffer[i].ElementWidth         = AUDIO_ELEMENT_WIDTH;
            audioBuffer[i].CallbackParameter    = &audioBuffer[i];
            audioBuffer[i].ProcessedFlag        = FALSE;
            audioBuffer[i].pNext                = (ADI_DEV_1D_BUFFER *)&audioBuffer[i + 1];
            }
        audioBuffer[i-1].pNext = (ADI_DEV_1D_BUFFER *)NULL;  // terminate the chain of buffers


        // initialize audio buffer queue
        QueueInit(&audioQueue, audioQueueEntry, AUDIO_BUFFER_COUNT);


        // open the audio input driver for inbound data flow
        result = adi_dev_Open(
            devManagerHandle,           // device manager handle
            &ADISPORTEntryPoint,        // entry point
            0,                          // device number
            &audioInHandle,             // client handle
            &audioInHandle,             // device handle
            ADI_DEV_DIRECTION_INBOUND,  // direction
            dmaManagerHandle,           // DMA Handle
        #if defined(USE_DEFERRED_CALLBACKS)
            dcbManagerHandle,           // DCB Handle
        #else
            NULL,                       // DCB Handle
        #endif
            AudioInCallback);           // callback routine

        
        // configure the audio input driver
        ADI_DEV_CMD_VALUE_PAIR audioInConfig [] = {
            { ADI_DEV_CMD_SET_DATAFLOW_METHOD,      (void *)ADI_DEV_MODE_CHAINED_LOOPBACK   },
            { ADI_DEV_CMD_SET_STREAMING,            (void *)FALSE                           },
            { ADI_SPORT_CMD_SET_RCR1,               (void *)(RFSR | LRFS | RCKFE)           },
            { ADI_SPORT_CMD_SET_RCR2,               (void *)(15 | RSFSE)                    },
            { ADI_DEV_CMD_END,                      NULL                                    }
        };

        result = (adi_dev_Control(audioInHandle, ADI_DEV_CMD_TABLE, audioInConfig));

        // give the buffers to audio input driver
        result = adi_dev_Read(audioInHandle, ADI_DEV_1D, (ADI_DEV_BUFFER *)audioBuffer);

        // audio will be processed in the callback function
        // create thread to send audio packet over the network
        result = VDK_CreateThreadEx ( &audioSendTCB );

        #ifndef DISABLE_AUDIO
        // enable data flow on audio input driver
        result = adi_dev_Control(audioInHandle, ADI_DEV_CMD_SET_DATAFLOW, (void *)TRUE);
        #endif


        // look for keep alive messages
        while (1)
            {
            result = (u32)recv(audioSocketID, &keepAliveMessage, 1, 0);

            // reset the packet count if a keep alive message was received
            if (result > 0)
                {
                audioPacketCount = 0;
                }
            *pPORTFIO_TOGGLE = 0x0080;  // toggle LED 2
            }
        }
    }



int
audio_ErrorFunction(void **inPtr)
    {
    VDK_DestroyThread(VDK_GetThreadID(), false);
    return 0;
    }

void
audio_InitFunction(void **inPtr, VDK_ThreadCreationBlock *pTCB)
    {
    }

void
audio_DestroyFunction(void **inPtr)
    {
    }


